#pragma once
#include "CalendarDlg.h"
#include "BaseEdit.h"
#include <SQLTYPES.H>

namespace FilterEdit
{
class CDateTimeEdit : public CBaseEdit
{
public:
	enum e_Mode {eDate, eTime, eDateTime};

	CCalendarDlg *m_pCalendarDlg;

	CDateTimeEdit (const e_Mode eMode = eDate);
	virtual ~CDateTimeEdit ();

	void SetMode (const e_Mode eMode);
	e_Mode GetMode () const;
	bool SetDateTime (const COleDateTime &dt);
	bool GetDateTime (COleDateTime &dt);
	bool SetDate (const int iDay, const int iMonth, const int iYear);
	bool GetDate (int &iDay, int &iMonth, int &iYear);
	bool GetTimeStamp (SQL_TIMESTAMP_STRUCT *ts_);
	int GetDay ();
	int GetMonth ();
	int GetYear ();
	bool SetTime (const int iHour, const int iMinute, const int iSecond);
	bool GetTime (int &iHour, int &iMinute, int &iSecond);
	int GetHour ();
	int GetMinute ();
	int GetSecond ();
	void CreateToolTip (CWnd *pParent);
	virtual void Increment (const int iInc);

	void GetSel (int &nStartChar, int &nEndChar) const;

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDateTimeEdit)
protected:
	virtual void PreSubclassWindow ();
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CDateTimeEdit)
	afx_msg void OnKeyDown (UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnChar (UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDown (UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp (UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk (UINT nFlags, CPoint point);
	afx_msg void OnMouseMove (UINT nFlags, CPoint point);
	afx_msg void OnKillFocus (CWnd* pNewWnd);
	afx_msg void OnCalendar ();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

	virtual bool SyntaxCheck (const CString &strText);
	virtual bool SemanticCheck (const CString &strText,
		CString *pstrErrorMsg = 0);
	void FormatDateTime ();
	bool FormatDate (CString &strText);
	bool FormatDay (CString &strText);
	bool FormatMonth (CString &strText);
	bool FormatYear (CString &strText);
	bool FormatTime (CString &strText);
	bool IsDateValid (const CString &strText, CString *pstrErrorMsg = 0);
	bool IsDayMonthYearValid (const CString &strText,
		CString *pstrErrorMsg = 0);
	bool IsMonthDayYearValid (const CString &strText,
		CString *pstrErrorMsg = 0);
	bool IsYearMonthDayValid (const CString &strText,
		CString *pstrErrorMsg = 0);
	void MoveLeft ();
	void MoveRight ();

private:
	enum e_Part {eDay, eMonth, eYear, eHour, eMinute, eSecond};

	e_Mode m_eMode;
	DWORD m_dwMargins;
	CButton m_ButtonCalendar;
	bool m_bFromSpin;

	static bool m_bInitialised;
	static CRegEx m_TimeRegEx;
	static CRegEx m_DateRegEx;
	static CRegEx m_DateTimeRegEx;
	static DWORD m_dwCenturyPrefix;
	static DWORD m_dwTwoDigitYearMax;
	static TCHAR m_chFieldOrder;
	static int m_iDayIndexStart;
	static int m_iDayIndexEnd;
	static int m_iMonthIndexStart;
	static int m_iMonthIndexEnd;
	static int m_iYearIndexStart;
	static int m_iYearIndexEnd;
	static int m_iDateLength;
	int m_iHourIndexStart;
	int m_iHourIndexEnd;
	int m_iMinuteIndexStart;
	int m_iMinuteIndexEnd;
	int m_iSecondIndexStart;
	int m_iSecondIndexEnd;
	static TCHAR m_cDateSeparator;
	static TCHAR m_cTimeSeparator;
	static CString m_strDateToolTip;
	static CString m_strTimeToolTip;
	static CString m_strDateTimeToolTip;

	void SetTimeIndexes ();
	bool ValidateDayMonth (const int iDay, const int iMonth);
	bool ValidateLeapYear (const int iDay, const int iMonth,
		const int iYear);
	bool ExpandYear (CString &strYear);
	e_Part LocatePart (const int iSelStart, int &iStart, int &iEnd) const;
	e_Part LocateDate (const int iSelStart, int &iStart, int &iEnd) const;
	e_Part LocateTime (const int iSelStart, int &iStart, int &iEnd) const;
	int GetDay (const CString &strDate);
	int GetMonth (const CString &strDate);
	int GetYear (const CString &strDate);
	int MaxDay (const int iMonth, const int iYear);
};
}
